"""
Simple example that counts the number of changes in an osm diff file.

Shows how to detect the different kind of modifications.
"""
import osmium as o
import sys

class Stats(object):

    def __init__(self):
        self.added = 0
        self.modified = 0
        self.deleted = 0

    def add(self, o):
        if o.deleted:
            self.deleted += 1
        elif o.version == 1:
            self.added += 1
        else:
            self.modified += 1

    def outstats(self, prefix):
        print("%s added: %d" % (prefix, self.added))
        print("%s modified: %d" % (prefix, self.modified))
        print("%s deleted: %d" % (prefix, self.deleted))


class FileStatsHandler(o.SimpleHandler):
    def __init__(self):
        super(FileStatsHandler, self).__init__()
        self.nodes = Stats()
        self.ways = Stats()
        self.rels = Stats()

    def node(self, n):
        self.nodes.add(n)

    def way(self, w):
        self.ways.add(w)

    def relation(self, r):
        self.rels.add(r)


if __name__ == '__main__':
    if len(sys.argv) != 2:
        print("Usage: python osm_file_stats.py <osmfile>")
        sys.exit(-1)

    h = FileStatsHandler()

    h.apply_file(sys.argv[1])

    h.nodes.outstats("Nodes")
    h.ways.outstats("Ways")
    h.rels.outstats("Relations")



from nose.tools import *
import unittest
import os
import sys
from datetime import datetime

from helpers import create_osm_file, osmobj, check_repr, HandlerTestBase

import osmium as o

class TestTagEmptyTagListLength(HandlerTestBase, unittest.TestCase):
    data = "n234 x1 y2"

    class Handler(o.SimpleHandler):

        def node(self, n):
            assert_equals(0, len(n.tags))
            assert_false(n.tags)

class TestTagEmptyTagListContains(HandlerTestBase, unittest.TestCase):
    data = "n234 x1 y2"

    class Handler(o.SimpleHandler):

        def node(self, n):
            assert_not_in("a", n.tags)

class TestTagEmptyTagListGet(HandlerTestBase, unittest.TestCase):
    data = "n234 x1 y2"

    class Handler(o.SimpleHandler):

        def node(self, n):
            assert_equals(None, n.tags.get("foo"))
            assert_equals(None, n.tags.get("foo", None))
            assert_equals("fs", n.tags.get("foo", "fs"))

class TestTagEmptyTagListIndexOp(HandlerTestBase, unittest.TestCase):
    data = "n234 x1 y2"

    class Handler(o.SimpleHandler):

        def node(self, n):
            with assert_raises(KeyError):
                n.tags["foo"]
            with assert_raises(KeyError):
                n.tags[None]

class TestTagListLen(HandlerTestBase, unittest.TestCase):
    data = """\
           n1 x0 y0 Ta=a
           n2 Tkey=value
           n3 Tfoo=1,bar=2,foobar=33
           """
    class Handler(o.SimpleHandler):

        expected_len = { 1 : 1, 2 : 1, 3 : 3}

        def node(self, n):
            assert_true(n.tags)
            assert_equals(self.expected_len[n.id], len(n.tags))

class TestTagContains(HandlerTestBase, unittest.TestCase):
    data = "n234 Tabba=x,2=vvv,xx=abba"

    class Handler(o.SimpleHandler):

        def node(self, n):
            assert_in("abba", n.tags)
            assert_in("2", n.tags)
            assert_in("xx", n.tags)
            assert_not_in("x", n.tags)
            assert_not_in(None, n.tags)
            assert_not_in("", n.tags)
            assert_true(check_repr(n.tags))

class TestTagIndexOp(HandlerTestBase, unittest.TestCase):
    data = "n234 Tabba=x,2=vvv,xx=abba"

    class Handler(o.SimpleHandler):

        def node(self, n):
            eq_("x", n.tags["abba"])
            eq_("vvv", n.tags["2"])
            eq_("abba", n.tags["xx"])
            for k in ("x", "addad", "..", None):
                with assert_raises(KeyError):
                    n.tags[k]

class TestTagGet(HandlerTestBase, unittest.TestCase):
    data = "n234 Tabba=x,2=vvv,xx=abba"

    class Handler(o.SimpleHandler):

        def node(self, n):
            eq_("x", n.tags.get("abba"))
            eq_("vvv", n.tags.get("2", None))
            eq_("abba", n.tags.get("xx", "ff"))
            eq_("43 fg", n.tags.get("_", "43 fg"))
            assert_is_none(n.tags.get("gerger4"))
            assert_is_none(n.tags.get("ffleo", None))



""" Provides some helper functions for test.
"""
import random
import tempfile
import os
import sys
from textwrap import dedent
import osmium
from datetime import datetime

if sys.version_info[0] == 3:
    from datetime import timezone

    def mkdate(*args):
        return datetime(*args, tzinfo=timezone.utc)
else:
    def mkdate(*args):
        return datetime(*args)


def _complete_object(o):
    """Takes a hash with an incomplete OSM object description and returns a
       complete one.
    """
    if o['type'] == 'C':
        ret = { 'created_at' : "2005-04-09T19:54:13Z",
                'num_changes' : 2, 'closed_at' : "2005-04-09T20:54:39Z",
                'open' : "false", 'min_lon' : -0.1465242,
                'min_lat' : 51.5288506, 'max_lon' : -0.1464925,
                'max_lat' : 51.5288620, 'user' : "Steve", 'uid' : "1",
                'tags' : None
        }
    else:
        ret = { 'version' : '1', 'timestamp': "2012-05-01T15:06:20Z",
                'changeset' : "11470653", 'uid' : "122294", 'user' : "foo",
                'tags' : {}
        }
    ret.update(o)
    if ret['type'] == 'N':
        if 'lat' not in ret:
            ret['lat'] = random.random()*180 - 90
        if 'lon' not in ret:
            ret['lon'] = random.random()*360 - 180
    return ret

def _write_osm_obj(fd, obj):
    if obj['type'] == 'N':
        fd.write(('<node id="%(id)d" lat="%(lat).8f" lon="%(lon).8f" version="%(version)s" timestamp="%(timestamp)s" changeset="%(changeset)s" uid="%(uid)s" user="%(user)s"'% obj).encode('utf-8'))
        if obj['tags'] is None:
            fd.write('/>\n'.encode('utf-8'))
        else:
            fd.write('>\n'.encode('utf-8'))
            for k,v in iter(obj['tags'].items()):
                fd.write(('  <tag k="%s" v="%s"/>\n' % (k, v)).encode('utf-8'))
            fd.write('</node>\n'.encode('utf-8'))
    elif obj['type'] == 'W':
        fd.write(('<way id="%(id)d" version="%(version)s" changeset="%(changeset)s" timestamp="%(timestamp)s" user="%(user)s" uid="%(uid)s">\n' % obj).encode('utf-8'))
        for nd in obj['nodes']:
            fd.write(('<nd ref="%s" />\n' % (nd,)).encode('utf-8'))
        for k,v in iter(obj['tags'].items()):
            fd.write(('  <tag k="%s" v="%s"/>\n' % (k, v)).encode('utf-8'))
        fd.write('</way>\n'.encode('utf-8'))
    elif obj['type'] == 'R':
        fd.write(('<relation id="%(id)d" version="%(version)s" changeset="%(changeset)s" timestamp="%(timestamp)s" user="%(user)s" uid="%(uid)s">\n' % obj).encode('utf-8'))
        for mem in obj['members']:
            fd.write(('  <member type="%s" ref="%s" role="%s"/>\n' % mem).encode('utf-8'))
        for k,v in iter(obj['tags'].items()):
            fd.write(('  <tag k="%s" v="%s"/>\n' % (k, v)).encode('utf-8'))
        fd.write('</relation>\n'.encode('utf-8'))
    elif obj['type'] == 'C':
        fd.write(('<changeset id="%(id)d" created_at="%(created_at)s" num_changes="%(num_changes)d" closed_at="%(closed_at)s" open="%(open)s" min_lon="%(min_lon).8f" min_lat="%(min_lat).8f" max_lon="%(max_lon).8f" max_lat="%(max_lat).8f"  user="%(user)s" uid="%(uid)s"' % obj).encode('utf-8'))
        if obj['tags'] is None:
            fd.write('/>\n'.encode('utf-8'))
        else:
            fd.write('>\n'.encode('utf-8'))
            for k,v in iter(obj['tags'].items()):
                fd.write(('  <tag k="%s" v="%s"/>\n' % (k, v)).encode('utf-8'))
            fd.write('</changeset>\n'.encode('utf-8'))



def create_osm_file(data):
    """Creates a temporary osm XML file. The data is a list of OSM objects,
       each described by a hash of attributes. Most attributes are optional
       and will be filled with sensitive values, if missing. Mandatory are
       only `type` and `id`. For ways, nodes are obligatory and for relations
       the memberlist.
    """
    data.sort(key=lambda x:('NWR'.find(x['type']), x['id']))
    with tempfile.NamedTemporaryFile(dir=tempfile.gettempdir(), suffix='.osm', delete=False) as fd:
        fname = fd.name
        fd.write("<?xml version='1.0' encoding='UTF-8'?>\n".encode('utf-8'))
        fd.write('<osm version="0.6" generator="test-pyosmium" timestamp="2014-08-26T20:22:02Z">\n'.encode('utf-8'))
        fd.write('\t<bounds minlat="-90" minlon="-180" maxlat="90" maxlon="180"/>\n'.encode('utf-8'))

        for obj in data:
            _write_osm_obj(fd, _complete_object(obj))

        fd.write('</osm>\n'.encode('utf-8'))

    return fname

def create_opl_file(data):
    with tempfile.NamedTemporaryFile(dir=tempfile.gettempdir(), suffix='.opl', delete=False) as fd:
        fname = fd.name
        fd.write(dedent(data).encode('utf-8'))
        fd.write(b'\n')

    return fname


def osmobj(kind, **args):
    ret = dict(args)
    ret['type'] = kind
    return ret

def check_repr(o):
    return not str(o).startswith('<') and not repr(o).startswith('<')

class HandlerTestBase:

    apply_locations = False
    apply_idx = 'flex_mem'

    def test_func(self):
        if isinstance(self.data, (list, tuple)):
            fn = create_osm_file(self.data)
        else:
            fn = create_opl_file(self.data)

        try:
            self.handler = self.Handler()
            self.handler.apply_file(fn, self.apply_locations, self.apply_idx)
        finally:
            os.remove(fn)

        if hasattr(self, "check_result"):
            self.check_result()


class CountingHandler(osmium.SimpleHandler):

    def __init__(self):
        super(CountingHandler, self).__init__()
        self.counts = [0, 0, 0, 0]

    def node(self, _):
        self.counts[0] += 1

    def way(self, _):
        self.counts[1] += 1

    def relation(self, _):
        self.counts[2] += 1

    def area(self, _):
        self.counts[3] += 1


class HandlerFunction(osmium.SimpleHandler):

    def __init__(self, **kwargs):
        super(HandlerFunction, self).__init__()

        for cb in ('node', 'way', 'relation', 'area'):
            if cb in kwargs:
                setattr(self, cb, kwargs[cb])

    def run(self, data, apply_locations=False, apply_idx='flex_mem'):
        if isinstance(data, (list, tuple)):
            fn = create_osm_file(data)
        else:
            fn = create_opl_file(data)

        try:
            self.apply_file(fn, apply_locations, apply_idx)
        finally:
            os.remove(fn)




"""
This also creates stats over an OSM file, only the file is first read into
a string buffer.

Shows how to use input from strings.
"""
import osmium as o
import sys
try:
    import urllib.request as urlrequest
except ImportError:
    import urllib2 as urlrequest

class FileStatsHandler(o.SimpleHandler):
    def __init__(self):
        super(FileStatsHandler, self).__init__()
        self.nodes = 0
        self.ways = 0
        self.rels = 0

    def node(self, n):
        self.nodes += 1

    def way(self, w):
        self.ways += 1

    def relation(self, r):
        self.rels += 1


if __name__ == '__main__':
    if len(sys.argv) != 2:
        print("Usage: python osm_url_stats.py <osmfile>")
        sys.exit(-1)


    data = urlrequest.urlopen(sys.argv[1]).read()

    h = FileStatsHandler()
    h.apply_buffer(data, sys.argv[1])

    print("Nodes: %d" % h.nodes)
    print("Ways: %d" % h.ways)
    print("Relations: %d" % h.rels)



from nose.tools import *
import unittest
import tempfile
import os
from contextlib import contextmanager
from datetime import datetime
from collections import OrderedDict
import logging
import sys

import osmium as o

log = logging.getLogger(__name__)

if sys.version_info[0] == 3:
    from datetime import timezone

    def mkdate(*args):
        return datetime(*args, tzinfo=timezone.utc)
else:
    def mkdate(*args):
        return datetime(*args)

@contextmanager
def WriteExpect(expected):
    fname = tempfile.mktemp(dir=tempfile.gettempdir(), suffix='.opl')
    writer = o.SimpleWriter(fname, 1024*1024)
    try:
        yield writer
    finally:
        writer.close()

    with open(fname, 'r') as fd:
        line = fd.readline().strip()
    assert_equals(line, expected)
    os.remove(fname)

class O(object):
    def __init__(self, **params):
        for k,v in params.items():
            setattr(self, k, v)

class TestWriteSimpleAttributes(unittest.TestCase):

    test_data_simple_attr = (
      (O(id=None), '0 v0 dV c0 t i0 u T'),
      (O(visible=None), '0 v0 dV c0 t i0 u T'),
      (O(version=None), '0 v0 dV c0 t i0 u T'),
      (O(uid=None), '0 v0 dV c0 t i0 u T'),
      (O(user=None), '0 v0 dV c0 t i0 u T'),
      (O(timestamp=None), '0 v0 dV c0 t i0 u T'),
      (O(id=1), '1 v0 dV c0 t i0 u T'),
      (O(id=-99), '-99 v0 dV c0 t i0 u T'),
      (O(visible=True), '0 v0 dV c0 t i0 u T'),
      (O(visible=False), '0 v0 dD c0 t i0 u T'),
      (O(version=23), '0 v23 dV c0 t i0 u T'),
      (O(user="Schmidt"), '0 v0 dV c0 t i0 uSchmidt T'),
      (O(user=""), '0 v0 dV c0 t i0 u T'),
      (O(uid=987), '0 v0 dV c0 t i987 u T'),
      (O(timestamp='2012-04-14T20:58:35Z'), '0 v0 dV c0 t2012-04-14T20:58:35Z i0 u T'),
      (O(timestamp=mkdate(2009, 4, 14, 20, 58, 35)), '0 v0 dV c0 t2009-04-14T20:58:35Z i0 u T'),
      (O(timestamp='1970-01-01T00:00:01Z'), '0 v0 dV c0 t1970-01-01T00:00:01Z i0 u T'),
    )

    def test_node_simple_attr(self):
        for node, out in self.test_data_simple_attr:
            with WriteExpect('n' + out + ' x y') as w:
                w.add_node(node)

    def test_way_simple_attr(self):
        for way, out in self.test_data_simple_attr:
            with WriteExpect('w' + out + ' N') as w:
                w.add_way(way)

    def test_relation_simple_attr(self):
        for rel, out in self.test_data_simple_attr:
            with WriteExpect('r' + out + ' M') as w:
                w.add_relation(rel)

class TestWriteTags(unittest.TestCase):

    test_data_tags = (
     (None, 'T'),
     ([], 'T'),
     ({}, 'T'),
     ((("foo", "bar"), ), 'Tfoo=bar'),
     ((("foo", "bar"), ("2", "1")), 'Tfoo=bar,2=1'),
     ({'test' : 'drive'}, 'Ttest=drive'),
     (OrderedDict((('a', 'b'), ('c', '3'))), 'Ta=b,c=3'),
    )

    def test_node_tags(self):
        for tags, out in self.test_data_tags:
            with WriteExpect('n0 v0 dV c0 t i0 u ' + out + ' x y') as w:
                w.add_node(O(tags=tags))

    def test_way_tags(self):
        for tags, out in self.test_data_tags:
            with WriteExpect('w0 v0 dV c0 t i0 u ' + out + ' N') as w:
                w.add_way(O(tags=tags))

    def test_relation_tags(self):
        for tags, out in self.test_data_tags:
            with WriteExpect('r0 v0 dV c0 t i0 u ' + out + ' M') as w:
                w.add_relation(O(tags=tags))


class TestWriteNode(unittest.TestCase):

    def test_location_tuple(self):
        with WriteExpect('n0 v0 dV c0 t i0 u T x1.1234561 y0.1234561') as w:
            w.add_node(O(location=(1.1234561, 0.1234561)))

    def test_location_rounding(self):
        with WriteExpect('n0 v0 dV c0 t i0 u T x30.46 y50.37') as w:
            w.add_node(O(location=(30.46, 50.37)))

    def test_location_none(self):
        with WriteExpect('n0 v0 dV c0 t i0 u T x y') as w:
            w.add_node(O(location=None))

class TestWriteWay(unittest.TestCase):

    def test_node_list(self):
        with WriteExpect('w0 v0 dV c0 t i0 u T Nn1,n2,n3,n-4') as w:
            w.add_way(O(nodes=(1, 2, 3, -4)))

    def test_node_list_none(self):
        with WriteExpect('w0 v0 dV c0 t i0 u T N') as w:
            w.add_way(O(nodes=None))

class TestWriteRelation(unittest.TestCase):

    def test_relation_members(self):
        with WriteExpect('r0 v0 dV c0 t i0 u T Mn34@foo,r200@,w1111@x') as w:
            w.add_relation(O(members=(('n', 34, 'foo'),
                                      ('r', 200, ''),
                                      ('w', 1111, 'x')
                                     )))

    def test_relation_members_None(self):
        with WriteExpect('r0 v0 dV c0 t i0 u T M') as w:
            w.add_relation(O(members=None))

if __name__ == '__main__':
    unittest.main()



import sys
import sysconfig
import os

# use some black magic to find the libraries in the build directory
# borrowed from http://stackoverflow.com/questions/14320220/testing-python-c-libraries-get-build-path

build_dir = "../../build/lib.%s-%d.%d" % (
                    sysconfig.get_platform(),
                    sys.version_info[0], sys.version_info[1])

# insert after the current directory
sys.path.insert(1, os.path.normpath(os.path.join(os.path.realpath(__file__), build_dir)))

import nose
nose.main()



"""
Converts a file from one format to another.

This example shows how to write objects to a file.
"""

import osmium as o

import sys

class Convert(o.SimpleHandler):

    def __init__(self, writer):
        super(Convert, self).__init__()
        self.writer = writer

    def node(self, n):
        self.writer.add_node(n)

    def way(self, w):
        self.writer.add_way(w)

    def relation(self, r):
        self.writer.add_relation(r)

if __name__ == '__main__':
    if len(sys.argv) != 3:
        print("Usage: python convert.py <infile> <outfile>")
        sys.exit(-1)

    writer = o.SimpleWriter(sys.argv[2])
    handler = Convert(writer)

    handler.apply_file(sys.argv[1])

    writer.close()





from nose.tools import *
import unittest
import os
import sys
from datetime import datetime

from helpers import create_osm_file, osmobj, HandlerTestBase

import osmium as o

class TestLength(HandlerTestBase, unittest.TestCase):
    data = """\
           r2 Mn3@
           r4
           r45 Mw1@fo,r45@4,r45@5
           """

    class Handler(o.SimpleHandler):
        expected_length = { 2 : 1, 4 : 0, 45 : 3 }

        def relation(self, r):
            assert_equals(self.expected_length[r.id], len(r.members))

class TestMembers(HandlerTestBase, unittest.TestCase):
    data = """r34 Mn23@,n12@foo,w5@.,r34359737784@()"""

    class Handler(o.SimpleHandler):

        def relation(self, r):
            m = list(r.members)
            eq_(4, len(m))
            eq_(23, m[0].ref)
            eq_('n', m[0].type)
            eq_('', m[0].role)
            eq_(12, m[1].ref)
            eq_('n', m[1].type)
            eq_('foo', m[1].role)
            eq_(5, m[2].ref)
            eq_('w', m[2].type)
            eq_('.', m[2].role)
            eq_(34359737784, m[3].ref)
            eq_('r', m[3].type)
            eq_('()', m[3].role)



from io import BytesIO
import os
from textwrap import dedent

from helpers import mkdate, osmobj, create_osm_file, CountingHandler
from nose.tools import *

try:
    from urllib.error import URLError
except ImportError:
    from urllib2 import URLError

try:
    from unittest.mock import MagicMock, patch
except ImportError:
    from mock import MagicMock, patch

import osmium as o
import osmium.replication.server as rserv
import osmium.replication
import tempfile
import datetime

class UrllibMock(MagicMock):

    def set_result(self, s):
        self.return_value = BytesIO(dedent(s).encode())

    def set_script(self, files):
        self.side_effect = [BytesIO(dedent(s).encode()) for s in files]

def test_get_state_url():
    svr = rserv.ReplicationServer("http://text.org")

    data = [
        (None,      'http://text.org/state.txt'),
        (1,         'http://text.org/000/000/001.state.txt'),
        (999,       'http://text.org/000/000/999.state.txt'),
        (1000,      'http://text.org/000/001/000.state.txt'),
        (573923,    'http://text.org/000/573/923.state.txt'),
        (3290012,   'http://text.org/003/290/012.state.txt'),
    ]

    for i, o in data:
        assert_equals(o, svr.get_state_url(i))

def test_get_diff_url():
    svr = rserv.ReplicationServer("https://who.is/me/")

    data = [
        (1,         'https://who.is/me//000/000/001.osc.gz'),
        (500,       'https://who.is/me//000/000/500.osc.gz'),
        (83750,     'https://who.is/me//000/083/750.osc.gz'),
        (999999999, 'https://who.is/me//999/999/999.osc.gz'),
    ]

    for i, o in data:
        assert_equals(o, svr.get_diff_url(i))

@patch('osmium.replication.server.urlrequest.urlopen', new_callable=UrllibMock)
def test_get_state_valid(mock):
    mock.set_result("""\
        #Sat Aug 26 11:04:04 UTC 2017
        txnMaxQueried=1219304113
        sequenceNumber=2594669
        timestamp=2017-08-26T11\:04\:02Z
        txnReadyList=
        txnMax=1219304113
        txnActiveList=1219303583,1219304054,1219304104""")

    res = rserv.ReplicationServer("http://test.io").get_state_info()

    assert_is_not_none(res)
    assert_equals(res.timestamp, mkdate(2017, 8, 26, 11, 4, 2))
    assert_equals(res.sequence, 2594669)

    assert_equal(mock.call_count, 1)

@patch('osmium.replication.server.urlrequest.urlopen')
def test_get_state_server_timeout(mock):
    mock.side_effect = URLError(reason='Mock')

    svr = rserv.ReplicationServer("http://test.io")
    assert_is_none(svr.get_state_info())

@patch('osmium.replication.server.urlrequest.urlopen', new_callable=UrllibMock)
def test_apply_diffs_count(mock):
    mock.set_script(("""\
        sequenceNumber=100
        timestamp=2017-08-26T11\:04\:02Z
    """, """
        n1
        w1
        r1
    """))
    svr = rserv.ReplicationServer("http://test.io", "opl")

    h = CountingHandler()
    assert_equals(100, svr.apply_diffs(h, 100, 10000))

    assert_equals(h.counts, [1, 1, 1, 0])

@patch('osmium.replication.server.urlrequest.urlopen', new_callable=UrllibMock)
def test_apply_diffs_without_simplify(mock):
    mock.set_script(("""\
        sequenceNumber=100
        timestamp=2017-08-26T11\:04\:02Z
    """, """
        n1 v23
        n1 v24
        w1
        r1
    """))
    svr = rserv.ReplicationServer("http://test.io", "opl")

    h = CountingHandler()
    assert_equals(100, svr.apply_diffs(h, 100, 10000, simplify=False))
    assert_equals([2, 1, 1, 0], h.counts)

@patch('osmium.replication.server.urlrequest.urlopen', new_callable=UrllibMock)
def test_apply_diffs_with_simplify(mock):
    mock.set_script(("""\
        sequenceNumber=100
        timestamp=2017-08-26T11\:04\:02Z
    """, """
        n1 v23
        n1 v24
        w1
        r1
    """))
    svr = rserv.ReplicationServer("http://test.io", "opl")

    h = CountingHandler()
    assert_equals(100, svr.apply_diffs(h, 100, 10000, simplify=True))
    assert_equals([1, 1, 1, 0], h.counts)

@patch('osmium.replication.server.urlrequest.urlopen', new_callable=UrllibMock)
def test_apply_with_location(mock):
    mock.set_script(("""\
        sequenceNumber=100
        timestamp=2017-08-26T11\:04\:02Z
    """, """
        n1 x10.0 y23.0
        w1 Nn1,n2
    """))
    svr = rserv.ReplicationServer("http://test.io", "opl")

    class Handler(CountingHandler):
        def way(self, w):
            self.counts[1] += 1
            assert_equals(2, len(w.nodes))
            assert_equals(1, w.nodes[0].ref)
            assert_equals(10, w.nodes[0].location.lon)
            assert_equals(23, w.nodes[0].location.lat)
            assert_equals(2, w.nodes[1].ref)
            assert_false(w.nodes[1].location.valid())

    h = Handler()
    assert_equals(100, svr.apply_diffs(h, 100, 10000, idx="flex_mem"))

    assert_equals(h.counts, [1, 1, 0, 0])



@patch('osmium.replication.server.urlrequest.urlopen', new_callable=UrllibMock)
def test_apply_reader_without_simplify(mock):
    mock.set_script(("""\
        sequenceNumber=100
        timestamp=2017-08-26T11\:04\:02Z
    """, """
        n1 v23
        n1 v24
        w1
        r1
    """))
    svr = rserv.ReplicationServer("http://test.io", "opl")

    h = CountingHandler()

    diffs = svr.collect_diffs(100, 100000)
    assert_is_not_none(diffs)

    diffs.reader.apply(h, simplify=False)
    assert_equals([2, 1, 1, 0], h.counts)

@patch('osmium.replication.server.urlrequest.urlopen', new_callable=UrllibMock)
def test_apply_reader_with_simplify(mock):
    mock.set_script(("""\
        sequenceNumber=100
        timestamp=2017-08-26T11\:04\:02Z
    """, """
        n1 v23
        n1 v24
        w1
        r1
    """))
    svr = rserv.ReplicationServer("http://test.io", "opl")

    h = CountingHandler()
    diffs = svr.collect_diffs(100, 100000)
    assert_is_not_none(diffs)

    diffs.reader.apply(h, simplify=True)
    assert_equals([1, 1, 1, 0], h.counts)

@patch('osmium.replication.server.urlrequest.urlopen', new_callable=UrllibMock)
def test_apply_reader_with_location(mock):
    mock.set_script(("""\
        sequenceNumber=100
        timestamp=2017-08-26T11\:04\:02Z
    """, """
        n1 x10.0 y23.0
        w1 Nn1,n2
    """))
    svr = rserv.ReplicationServer("http://test.io", "opl")

    class Handler(CountingHandler):
        def way(self, w):
            self.counts[1] += 1
            assert_equals(2, len(w.nodes))
            assert_equals(1, w.nodes[0].ref)
            assert_equals(10, w.nodes[0].location.lon)
            assert_equals(23, w.nodes[0].location.lat)
            assert_equals(2, w.nodes[1].ref)
            assert_false(w.nodes[1].location.valid())

    h = Handler()
    diffs = svr.collect_diffs(100, 100000)
    assert_is_not_none(diffs)

    diffs.reader.apply(h, idx="flex_mem")

    assert_equals(h.counts, [1, 1, 0, 0])

def test_get_newest_change_from_file():
    data = [osmobj('N', id=1, version=1, changeset=63965061, uid=8369524,
                   timestamp='2018-10-29T03:56:07Z', user='x')]
    fn = create_osm_file(data)


    try:
        val = osmium.replication.newest_change_from_file(fn)
        assert_equals(val, mkdate(2018, 10, 29, 3, 56, 7))
    finally:
        os.remove(fn)



import osmium as o
import sys

class WayHandler(o.SimpleHandler):

    def __init__(self, idx):
        super(WayHandler, self).__init__()
        self.idx = idx

    def way(self, w):
        for n in w.nodes:
            n.lat, n.lon # throws an exception if the coordinates are missing
            loc = idx.get(n.ref)
        print("%d %s" %(w.id, len(w.nodes)))

if len(sys.argv) != 3:
    print("Usage: python create_nodecache.py <osm file> <node cache>")
    exit()

reader = o.io.Reader(sys.argv[1], o.osm.osm_entity_bits.WAY)

idx = o.index.create_map("sparse_file_array," + sys.argv[2])
lh = o.NodeLocationsForWays(idx)
lh.ignore_errors()

o.apply(reader, lh, WayHandler(idx))

reader.close()



from nose.tools import *
import unittest
import os
import sys
from helpers import create_osm_file, osmobj, check_repr, HandlerTestBase, mkdate

import osmium as o

class TestLocation(unittest.TestCase):

    def test_invalid_location(self):
        loc = o.osm.Location()
        assert_false(loc.valid())
        assert_true(check_repr(loc))

    def test_valid_location(self):
        loc = o.osm.Location(1,10)
        assert_equals(loc.lon, 1, 0.0001)
        assert_equals(loc.lat, 10, 0.00001)
        assert_equals(loc.x, 10000000)
        assert_equals(loc.y, 100000000)
        assert_true(check_repr(loc))


class TestNodeAttributes(HandlerTestBase, unittest.TestCase):
    data = [osmobj('N', id=1, version=5, changeset=58674, uid=42,
                   timestamp='2014-01-31T06:23:35Z', user='anonymous')]

    class Handler(o.SimpleHandler):
        def node(self, n):
            assert_equals(n.id, 1)
            assert_equals(n.deleted, False)
            assert_equals(n.visible, True)
            assert_equals(n.version, 5)
            assert_equals(n.changeset, 58674)
            assert_equals(n.uid, 42)
            assert_equals(n.user_is_anonymous(), False)
            assert_equals(n.timestamp, mkdate(2014, 1, 31, 6, 23, 35))
            assert_equals(n.user, 'anonymous')
            assert_equals(n.positive_id(), 1)
            assert_true(check_repr(n))


class TestNodePositiveId(HandlerTestBase, unittest.TestCase):
    data = [osmobj('N', id=-34, version=5, changeset=58674, uid=42,
                   timestamp='2014-01-31T06:23:35Z', user='anonymous')]

    class Handler(o.SimpleHandler):
        def node(self, n):
            assert_equals(n.positive_id(), 34)

class TestNodeLargeId(HandlerTestBase, unittest.TestCase):
    data = [osmobj('N', id=17179869418, version=5, changeset=58674, uid=42,
                   timestamp='2014-01-31T06:23:35Z', user='anonymous')]

    class Handler(o.SimpleHandler):
        def node(self, n):
            assert_equals(n.id, 17179869418)


class TestWayAttributes(HandlerTestBase, unittest.TestCase):

    apply_locations = True

    data = [osmobj('N', id=1, lat=0, lon=0),
            osmobj('N', id=3, lat=1, lon=1),
            osmobj('W', id=1, version=5, changeset=58674, uid=42,
                   timestamp='2014-01-31T06:23:35Z', user='anonymous',
                   nodes = [1,2,3])]

    class Handler(o.SimpleHandler):
        def way(self, n):
            assert_equals(n.id, 1)
            assert_equals(n.deleted, False)
            assert_equals(n.visible, True)
            assert_equals(n.version, 5)
            assert_equals(n.changeset, 58674)
            assert_equals(n.uid, 42)
            assert_equals(n.user_is_anonymous(), False)
            assert_equals(n.timestamp, mkdate(2014, 1, 31, 6, 23, 35))
            assert_equals(n.user, 'anonymous')
            assert_equals(n.positive_id(), 1)
            assert_false(n.is_closed())
            assert_false(n.ends_have_same_id())
            assert_false(n.ends_have_same_location())
            assert_true(check_repr(n))
            assert_true(check_repr(n.nodes))

class TestRelationAttributes(HandlerTestBase, unittest.TestCase):
    data = [osmobj('R', id=1, version=5, changeset=58674, uid=42,
                   timestamp='2014-01-31T06:23:35Z', user='anonymous',
                   members=[('way',1,'')])]

    class Handler(o.SimpleHandler):
        def relation(self, n):
            assert_equals(n.id, 1)
            assert_equals(n.deleted, False)
            assert_equals(n.visible, True)
            assert_equals(n.version, 5)
            assert_equals(n.changeset, 58674)
            assert_equals(n.uid, 42)
            assert_equals(n.user_is_anonymous(), False)
            assert_equals(n.timestamp, mkdate(2014, 1, 31, 6, 23, 35))
            assert_equals(n.user, 'anonymous')
            assert_equals(n.positive_id(), 1)
            assert_true(check_repr(n))
            assert_true(check_repr(n.members))

class TestAreaFromWayAttributes(HandlerTestBase, unittest.TestCase):
    data = [osmobj('N', id=1, lat=0, lon=0),
            osmobj('N', id=2, lat=0, lon=1),
            osmobj('N', id=3, lat=1, lon=0),
            osmobj('W', id=23, version=5, changeset=58674, uid=42,
                   timestamp='2014-01-31T06:23:35Z', user='anonymous',
                   nodes = [1,2,3,1], tags = { "area" : "yes" }),
           ]

    class Handler(o.SimpleHandler):
        def area(self, n):
            assert_equals(n.id, 46)
            assert_equals(n.deleted, False)
            assert_equals(n.visible, True)
            assert_equals(n.version, 5)
            assert_equals(n.changeset, 58674)
            assert_equals(n.uid, 42)
            assert_equals(n.user_is_anonymous(), False)
            assert_equals(n.timestamp, mkdate(2014, 1, 31, 6, 23, 35))
            assert_equals(n.user, 'anonymous')
            assert_equals(n.positive_id(), 46)
            assert_equals(n.orig_id(), 23)
            assert_equals(n.from_way(), True)
            assert_equals(n.is_multipolygon(), False)
            assert_equals(n.num_rings(), (1, 0))
            assert_equals(len(list(n.outer_rings())), 1)
            oring = list(n.outer_rings())[0]
            assert_equals(len(list(oring)), 4)
            assert_equals(set((1,2,3)), set([x.ref for x in oring]))
            assert_true(oring.is_closed())
            assert_true(oring.ends_have_same_id())
            assert_true(oring.ends_have_same_location())
            assert_equals(len(list(n.inner_rings(oring))), 0)

class TestChangesetAttributes(HandlerTestBase, unittest.TestCase):
    data = [osmobj('C', id=34, created_at="2005-04-09T19:54:13Z",
                num_changes=2, closed_at="2005-04-09T20:54:39Z",
                open="false", min_lon=-0.1465242,
                min_lat=51.5288506, max_lon=-0.1464925,
                max_lat=51.5288620, user="Steve", uid="1")
           ]

    class Handler(o.SimpleHandler):
        def changeset(self,c):
            assert_equals(34, c.id)
            assert_equals(1, c.uid)
            assert_false(c.user_is_anonymous())
            assert_equals("Steve", c.user)
            assert_equals(mkdate(2005, 4, 9, 19, 54, 13), c.created_at)
            assert_equals(mkdate(2005, 4, 9, 20, 54, 39), c.closed_at)
            assert_false(c.open)
            assert_equals(2, c.num_changes)
            assert_equals(0, len(c.tags))
            assert_equals(-1464925, c.bounds.top_right.x)
            assert_equals(515288620, c.bounds.top_right.y)
            assert_equals(-1465242, c.bounds.bottom_left.x)
            assert_equals(515288506, c.bounds.bottom_left.y)
            assert_true(check_repr(c))



from ._osm import *
import osmium.osm.mutable

def create_mutable_node(node, **args):
    """ Create a mutable node replacing the properties given in the
        named parameters. Note that this function only creates a shallow
        copy which is still bound to the scope of the original object.
    """
    return osmium.osm.mutable.Node(base=node, **args)

def create_mutable_way(way, **args):
    """ Create a mutable way replacing the properties given in the
        named parameters. Note that this function only creates a shallow
        copy which is still bound to the scope of the original object.
    """
    return osmium.osm.mutable.Way(base=way, **args)

def create_mutable_relation(rel, **args):
    """ Create a mutable relation replacing the properties given in the
        named parameters. Note that this function only creates a shallow
        copy which is still bound to the scope of the original object.
    """
    return osmium.osm.mutable.Relation(base=rel, **args)

Node.replace = create_mutable_node
Way.replace = create_mutable_way
Relation.replace = create_mutable_relation

Location.__repr__ = lambda l : 'osmium.osm.Location(x=%r, y=%r)' \
                               % (l.x, l.y) \
                               if l.valid() else 'osmium.osm.Location()'
Location.__str__ = lambda l : '%f/%f' % (l.lon_without_check(), l.lat_without_check()) \
                              if l.valid() else 'invalid'

Box.__repr__ = lambda b : 'osmium.osm.Box(bottom_left=%r, top_right=%r)' \
                          % (b.bottom_left, b.top_right)
Box.__str__ = lambda b : '(%s %s)' % (b.bottom_left, b.top_right)

Tag.__repr__ = lambda t : 'osmium.osm.Tag(k=%s, v=%s)' % (t.k, t.v)
Tag.__str__ = lambda t : '%s=%s' % (t.k, t.v)

TagList.__repr__ = lambda t : "osmium.osm.TagList({%s})" \
                              % ",".join(["%r=%r" % (i.k, i.v) for i in t])
TagList.__str__ = lambda t : "{%s}" % ",".join([str(i) for i in t])

NodeRef.__repr__ = lambda n : 'osmium.osm.NodeRef(ref=%r, location=%r)' % (n.ref, n.location)
NodeRef.__str__ = lambda n : '%s@%s' % (n.ref, n.location) if n.location.valid() \
                             else str(n.ref)

NodeRefList.__repr__ = lambda t : "%s([%s])" % (t.__class__.__name__,
                                                ",".join([repr(i) for i in t]))
NodeRefList.__str__ = lambda t : "[%s]" % ",".join([str(i) for i in t])

RelationMember.__repr__ = lambda r : 'osmium.osm.RelationMember(ref=%r, type=%r, role=%r)' \
                                     % (r.ref, r.type, r.role)
RelationMember.__str__ = lambda r : '%s%d@%s' % (r.type, r.ref, r.role) \
                                    if r.role else '%s%d' % (r.type, r.ref)

RelationMemberList.__repr__ = lambda t : "osmium.osm.RelationMemberList([%s])" \
                                  % ",".join([repr(i) for i in t])
RelationMemberList.__str__ = lambda t : "[%s]" % ",".join([str(i) for i in t])

OSMObject.__repr__ = lambda o : '%s(id=%r, deleted=%r, visible=%r, version=%r, changeset=%r, uid=%r, timestamp=%r, user=%r, tags=%r)'% (o.__class__.__name__, o.id, o.deleted, o.visible, o.version, o.changeset, o.uid, o.timestamp, o.user, o.tags)

def _str_ellipse(s, length=50):
    s = str(s)
    return s if len(s) <= length else (s[:length - 4] + '...' + s[-1])

Node.__repr__ = lambda o : '%s(id=%r, deleted=%r, visible=%r, version=%r, changeset=%r, uid=%r, timestamp=%r, user=%r, tags=%r, location=%r)'% (o.__class__.__name__, o.id, o.deleted, o.visible, o.version, o.changeset, o.uid, o.timestamp, o.user, o.tags, o.location)
Node.__str__ = lambda n : 'n%d: location=%s tags=%s' \
                          % (n.id, n.location, _str_ellipse(n.tags))

Way.__repr__ = lambda o : '%s(id=%r, deleted=%r, visible=%r, version=%r, changeset=%r, uid=%r, timestamp=%r, user=%r, tags=%r, nodes=%r)'% (o.__class__.__name__, o.id, o.deleted, o.visible, o.version, o.changeset, o.uid, o.timestamp, o.user, o.tags, o.nodes)
Way.__str__ = lambda o : 'w%d: nodes=%s tags=%s' \
                         % (o.id, _str_ellipse(o.nodes), _str_ellipse(o.tags))

Relation.__repr__ = lambda o : '%s(id=%r, deleted=%r, visible=%r, version=%r, changeset=%r, uid=%r, timestamp=%r, user=%r, tags=%r, members=%r)'% (o.__class__.__name__, o.id, o.deleted, o.visible, o.version, o.changeset, o.uid, o.timestamp, o.user, o.tags, o.members)
Relation.__str__ = lambda o : 'r%d: members=%s, tags=%s' \
                              % (o.id, _str_ellipse(o.members), _str_ellipse(o.tags))

Changeset.__repr__ = lambda o : '%s(id=%r, uid=%r, created_at=%r, closed_at=%r, open=%r, num_changes=%r, bounds=%r, user=%r, tags=%s)' %(o.__class__.__name__, o.id, o.uid, o.created_at, o.closed_at, o.open, o.num_changes, o.bounds, o.user, o.tags)
Changeset.__str__ = lambda o : 'c%d: closed_at=%s, bounds=%s, tags=%s' \
                               % (o.id, o.closed_at, o.bounds, _str_ellipse(o.tags))



"""
Simple example that counts the number of objects in an osm file.

Shows how to write a handler for the different types of objects.
"""
import osmium as o
import sys

class FileStatsHandler(o.SimpleHandler):

    def __init__(self):
        super(FileStatsHandler, self).__init__()
        self.nodes = 0
        self.ways = 0
        self.rels = 0

    def node(self, n):
        self.nodes += 1

    def way(self, w):
        self.ways += 1

    def relation(self, r):
        self.rels += 1


if __name__ == '__main__':
    if len(sys.argv) != 2:
        print("Usage: python osm_file_stats.py <osmfile>")
        sys.exit(-1)

    h = FileStatsHandler()

    h.apply_file(sys.argv[1])

    print("Nodes: %d" % h.nodes)
    print("Ways: %d" % h.ways)
    print("Relations: %d" % h.rels)



import osmium as o
import sys

if len(sys.argv) != 3:
    print("Usage: python create_nodecache.py <osm file> <node cache>")
    exit(-1)

reader = o.io.Reader(sys.argv[1], o.osm.osm_entity_bits.NODE)

idx = o.index.create_map("sparse_file_array," + sys.argv[2])
lh = o.NodeLocationsForWays(idx)

o.apply(reader, lh)

reader.close()



"""
Version information.
"""

# the major version
pyosmium_major = '2.15'
# current release (Pip version)
pyosmium_release = '2.15.0'

# libosmium version shipped with the Pip release
libosmium_version = '2.15.0'
# protozero version shipped with the Pip release
protozero_version = '1.6.4'
# pybind11 version shipped with the Pip release
pybind11_version = '2.2.4'



class OSMObject(object):
    """Mutable version of ``osmium.osm.OSMObject``. It exposes the following
       attributes ``id``, ``version``, ``visible``, ``changeset``, ``timestamp``,
       ``uid`` and ``tags``. Timestamps may be strings or datetime objects.
       Tags can be an osmium.osm.TagList, a dict-like object
       or a list of tuples, where each tuple contains a (key value) string pair.

       If the ``base`` parameter is given in the constructor, then the object
       will be initialised first from the attributes of this base object.
    """

    def __init__(self, base=None, id=None, version=None, visible=None, changeset=None,
            timestamp=None, uid=None, tags=None):
        if base is None:
            self.id = id
            self.version = version
            self.visible = visible
            self.changeset = changeset
            self.timestamp = timestamp
            self.uid = uid
            self.tags = tags
        else:
            self.id = base.id if id is None else id
            self.version = base.version if version is None else version
            self.visible = base.visible if visible is None else visible
            self.changeset = base.changeset if changeset is None else changeset
            self.timestamp = base.timestamp if timestamp is None else timestamp
            self.uid = base.uid if uid is None else uid
            self.tags = base.tags if tags is None else tags


class Node(OSMObject):
    """The mutable version of ``osmium.osm.Node``. It inherits all attributes
       from osmium.osm.mutable.OSMObject and adds a `location` attribute. This
       may either be an `osmium.osm.Location` or a tuple of lon/lat coordinates.
    """

    def __init__(self, base=None, location=None, **attrs):
        OSMObject.__init__(self, base=base, **attrs)
        if base is None:
            self.location = location
        else:
            self.location = location if location is not None else base.location


class Way(OSMObject):
    """The mutable version of ``osmium.osm.Way``. It inherits all attributes
       from osmium.osm.mutable.OSMObject and adds a `nodes` attribute. This may
       either be and ``osmium.osm.NodeList`` or a list consisting of
       ``osmium.osm.NodeRef`` or simple node ids.
    """

    def __init__(self, base=None, nodes=None, **attrs):
        OSMObject.__init__(self, base=base, **attrs)
        if base is None:
            self.nodes = nodes
        else:
            self.nodes = nodes if nodes is not None else base.nodes

class Relation(OSMObject):
    """The mutable version of ``osmium.osm.Relation``. It inherits all attributes
       from osmium.osm.mutable.OSMObject and adds a `members` attribute. This
       may either be an ``osmium.osm.RelationMemberList`` or a list consisting
       of ``osmium.osm.RelationMember`` or tuples of (type, id, role). The
       member type should be a single character 'n', 'w' or 'r'.
    """

    def __init__(self, base=None, members=None, **attrs):
        OSMObject.__init__(self, base=base, **attrs)
        if base is None:
            self.members = members
        else:
            self.members = members if members is not None else base.members





from nose.tools import *
import unittest
import os

from helpers import create_osm_file, osmobj

import osmium as o

class TestReaderFromFile(unittest.TestCase):

    def _run_file(self, fn):
        try:
            rd = o.io.Reader(fn)
            o.apply(rd, o.SimpleHandler())
            rd.close()
        finally:
            os.remove(fn)

    def test_node_only(self):
        self._run_file(create_osm_file([osmobj('N', id=1)]))

    def test_way_only(self):
        self._run_file(create_osm_file([osmobj('W', id=1, nodes=[1,2,3])]))

    def test_relation_only(self):
        self._run_file(create_osm_file([osmobj('R', id=1, members=[('way', 1, '')])]))

    def test_node_with_tags(self):
        self._run_file(create_osm_file([osmobj('N', id=1, 
                                               tags=dict(foo='bar', name='xx'))]))

    def test_way_with_tags(self):
        self._run_file(create_osm_file([osmobj('W', id=1, nodes=[1,2,3],
                                               tags=dict(foo='bar', name='xx'))]))

    def test_relation_with_tags(self):
        self._run_file(create_osm_file([osmobj('R', id=1, members=[('way', 1, '')],
                                               tags=dict(foo='bar', name='xx'))]))

    def test_broken_timestamp(self):
        fn = create_osm_file([osmobj('N', id=1, timestamp='x')])
        try:
            rd = o.io.Reader(fn)
            with assert_raises(ValueError):
                o.apply(rd, o.SimpleHandler())
                rd.close()
        finally:
            os.remove(fn)


class TestFileHeader(unittest.TestCase):

    def test_file_header(self):
        fn = create_osm_file([osmobj('N', id=1)])
        try:
            rd = o.io.Reader(fn)
            h = rd.header()
            assert_false(h.has_multiple_object_versions)
            assert_true(h.box().valid())
            assert_equals(h.box().size(), 64800.0)
            rd.close()
        finally:
            os.remove(fn)

if __name__ == '__main__':
    unittest.main()



from nose.tools import *
import unittest

import osmium as o

class TestIndexMapTypes(unittest.TestCase):

    def test_list_types(self):
        ml = o.index.map_types()
        assert_true(isinstance(ml, list))
        assert_greater(len(ml), 0)


class TestLocationTable(unittest.TestCase):

    def setUp(self):
        self.table = o.index.create_map("flex_mem")

    def test_set_get(self):
        self.table.set(4, o.osm.Location(3.4, -5.6))
        l = self.table.get(4)
        assert_almost_equal(3.4, l.lon, 5)
        assert_almost_equal(-5.6, l.lat, 5)

    def test_get_unset(self):
        with assert_raises(KeyError):
            self.table.get(56)

    def test_set_negative(self):
        with assert_raises(TypeError):
            self.table.set(-4, o.osm.Location(3.4, -5.6))

    def test_used_memory(self):
        self.table.set(4, o.osm.Location(3.4, -5.6))
        assert_greater(self.table.used_memory(), 0)

    def test_clear(self):
        self.table.set(593, o.osm.Location(0.35, 45.3))
        self.table.get(593)
        self.table.clear()
        with assert_raises(KeyError):
            self.table.get(593)




"""
This example shows how to filter and modify tags and write the results back.
It changes all tag values 'yes/no' to '1/0'.
"""

import osmium as o
import sys

class BoolNormalizer(o.SimpleHandler):

    def __init__(self, writer):
        super(BoolNormalizer, self).__init__()
        self.writer = writer

    def normalize(self, o):
        # if there are no tags we are done
        if not o.tags:
            return o

        # new tags should be kept in a list so that the order is preserved
        newtags = []
        # pyosmium is much faster writing an original osmium object than
        # a osmium.mutable.*. Therefore, keep track if the tags list was
        # actually changed.
        modified = False
        for t in o.tags:
            if t.v == 'yes':
                # custom tags should be added as a key/value tuple
                newtags.append((t.k, '1'))
                modified = True
            elif t.v == 'no':
                newtags.append((t.k, '0'))
                modified = True
            else:
                # if the tag is not modified, simply readd it to the list
                newtags.append(t)

        if modified:
            # We have changed tags. Create a new object as a copy of the
            # original one with the tag list replaced.
            return o.replace(tags=newtags)
        else:
            # Nothing changed, so simply return the original object
            # and discard the tag list we just created.
            return o

    def node(self, o):
        self.writer.add_node(self.normalize(o))

    def way(self, o):
        self.writer.add_way(self.normalize(o))

    def relation(self, o):
        self.writer.add_relation(self.normalize(o))


if __name__ == '__main__':
    if len(sys.argv) != 3:
        print("Usage: python normalize_boolean.py <infile> <outfile>")
        sys.exit(-1)


    writer = o.SimpleWriter(sys.argv[2])
    BoolNormalizer(writer).apply_file(sys.argv[1])

    writer.close()



"""
Filter all objects with a coastline tag.

This example shows how to write objects to a file.

We need to go twice over the file. First read the ways, filter the ones
we are interested in and remember the nodes required. Then, in a second
run all the relevant nodes and ways are written out.
"""

import osmium as o
import sys

class WayFilter(o.SimpleHandler):

    def __init__(self):
        super(WayFilter, self).__init__()
        self.nodes = set()

    def way(self, w):
        if 'natural' in w.tags and w.tags['natural'] == 'coastline':
            for n in w.nodes:
                self.nodes.add(n.ref)


class CoastlineWriter(o.SimpleHandler):

    def __init__(self, writer, nodes):
        super(CoastlineWriter, self).__init__()
        self.writer = writer
        self.nodes = nodes

    def node(self, n):
        if n.id in self.nodes:
            self.writer.add_node(n)

    def way(self, w):
        if 'natural' in w.tags and w.tags['natural'] == 'coastline':
            self.writer.add_way(w)


if __name__ == '__main__':
    if len(sys.argv) != 3:
        print("Usage: python filter_coastlines.py <infile> <outfile>")
        sys.exit(-1)


    # go through the ways to find all relevant nodes
    ways = WayFilter()
    ways.apply_file(sys.argv[1])

    # go through the file again and write out the data
    writer = o.SimpleWriter(sys.argv[2])
    CoastlineWriter(writer, ways.nodes).apply_file(sys.argv[1])

    writer.close()



from nose.tools import *
import unittest
import os
import sys
from datetime import datetime

from helpers import create_osm_file, osmobj, HandlerTestBase

import osmium as o

class TestLength(HandlerTestBase, unittest.TestCase):
    data = """\
           w593
           w4 Nn1,n2,n-34
           w8 Nn12,n12,n12,n0
           """

    class Handler(o.SimpleHandler):
        expected_length = { 593 : 0, 4 : 3, 8 : 4 }

        def way(self, w):
            assert_equals(self.expected_length[w.id], len(w.nodes))

class TestNodeIds(HandlerTestBase, unittest.TestCase):
    data = """w4 Nn1,n1,n34359737784,n-34,n0"""

    class Handler(o.SimpleHandler):

        def way(self, w):
            eq_(1, w.nodes[0].ref)
            eq_(1, w.nodes[1].ref)
            eq_(34359737784, w.nodes[2].ref)
            eq_(-34, w.nodes[3].ref)
            eq_(0, w.nodes[4].ref)
            eq_(0, w.nodes[-1].ref)

class TestMissingRef(HandlerTestBase, unittest.TestCase):
    data = """\
           n1 x0.5 y10.0
           w4 Nn1
           """

    class Handler(o.SimpleHandler):

        def way(self, w):
            eq_(1, w.nodes[0].ref)
            assert_false(w.nodes[0].location.valid())
            with assert_raises(o.InvalidLocationError):
                w.nodes[0].location.lat
            with assert_raises(o.InvalidLocationError):
                w.nodes[0].location.lon

class TestValidRefs(HandlerTestBase, unittest.TestCase):
    data = """\
           n1 x0.5 y10.0
           w4 Nn1
           """
    apply_locations = True

    class Handler(o.SimpleHandler):

        def way(self, w):
            eq_(1, w.nodes[0].ref)
            assert_true(w.nodes[0].location.valid())
            assert_almost_equal(w.nodes[0].location.lat, 10.0)
            assert_almost_equal(w.nodes[0].location.lon, 0.5)



"""
Search for pubs in an osm file and list their names.
"""
import osmium
import sys

class NamesHandler(osmium.SimpleHandler):

    def output_pubs(self, tags):
        if tags.get('amenity') == 'pub':
            if 'name' in tags:
                print(tags['name'])

    def node(self, n):
        self.output_pubs(n.tags)

    def way(self, w):
        self.output_pubs(w.tags)

if __name__ == '__main__':
    if len(sys.argv) != 2:
        print("Usage: python pub_names.py <osmfile>")
        sys.exit(-1)

    NamesHandler().apply_file(sys.argv[1])




"""
Compute the total length of highways in an osm file.

Shows how extract the geometry of a way.
"""
import osmium as o
import sys

class RoadLengthHandler(o.SimpleHandler):
    def __init__(self):
        super(RoadLengthHandler, self).__init__()
        self.length = 0.0

    def way(self, w):
        if 'highway' in w.tags:
            try:
                self.length += o.geom.haversine_distance(w.nodes)
            except o.InvalidLocationError:
                # A location error might occur if the osm file is an extract
                # where nodes of ways near the boundary are missing.
                print("WARNING: way %d incomplete. Ignoring." % w.id)

if __name__ == '__main__':
    if len(sys.argv) != 2:
        print("Usage: python road_length.py <osmfile>")
        sys.exit(-1)

    h = RoadLengthHandler()
    # As we need the geometry, the node locations need to be cached. Therefore
    # set 'locations' to true.
    h.apply_file(sys.argv[1], locations=True)

    print('Total way length: %.2f km' % (h.length/1000))



import os
import re
import sys
import platform
import subprocess

from setuptools import setup, Extension
from setuptools.command.build_ext import build_ext
from setuptools.command.sdist import sdist as orig_sdist
from distutils.version import LooseVersion

BASEDIR = os.path.split(os.path.abspath(__file__))[0]

class Pyosmium_sdist(orig_sdist):

    contrib = (
        ('libosmium', 'https://github.com/osmcode/libosmium/archive/v{}.tar.gz'),
        ('protozero', 'https://github.com/mapbox/protozero/archive/v{}.tar.gz'),
        ('pybind11', 'https://github.com/pybind/pybind11/archive/v{}.tar.gz'),
    )

    def make_release_tree(self, base_dir, files):
        orig_sdist.make_release_tree(self, base_dir, files)

        # add additional dependecies in the required version
        for name, tar_src in self.contrib:
            tarball = tar_src.format(versions[name + '_version'])
            print("Downloading and adding {} sources from {}".format(name, tarball))
            subprocess.call('wget -O - -q {} | tar xz -C {} --one-top-level=contrib/{} --strip-components=1'.format(
                   tarball, base_dir, name), shell=True)


def get_versions():
    """ Read the version file.

        The file cannot be directly imported because it is not installed
        yet.
    """
    version_py = os.path.join(BASEDIR, "src/osmium/version.py")
    v = {}
    with open(version_py) as version_file:
        # Execute the code in version.py.
        exec(compile(version_file.read(), version_py, 'exec'), v)

    return v

class CMakeExtension(Extension):
    def __init__(self, name, sourcedir=''):
        Extension.__init__(self, name, sources=[])
        self.sourcedir = os.path.abspath(sourcedir)


class CMakeBuild(build_ext):
    def run(self):
        try:
            out = subprocess.check_output(['cmake', '--version'])
        except OSError:
            raise RuntimeError("CMake must be installed to build the following extensions: " +
                               ", ".join(e.name for e in self.extensions))

        if platform.system() == "Windows":
            cmake_version = LooseVersion(re.search(r'version\s*([\d.]+)', out.decode()).group(1))
            if cmake_version < '3.1.0':
                raise RuntimeError("CMake >= 3.1.0 is required on Windows")

        for ext in self.extensions:
            self.build_extension(ext)

    def build_extension(self, ext):
        extdir = os.path.abspath(os.path.dirname(self.get_ext_fullpath(ext.name)))
        cmake_args = ['-DCMAKE_LIBRARY_OUTPUT_DIRECTORY=' + extdir,
                      '-DPYTHON_EXECUTABLE=' + sys.executable]

        cfg = 'Debug' if self.debug else 'Release'
        build_args = ['--config', cfg]

        if platform.system() == "Windows":
            cmake_args += ['-DCMAKE_LIBRARY_OUTPUT_DIRECTORY_{}={}'.format(cfg.upper(), extdir)]
            if sys.maxsize > 2**32:
                cmake_args += ['-A', 'x64']
            build_args += ['--', '/m']
        else:
            cmake_args += ['-DCMAKE_BUILD_TYPE=' + cfg]
            build_args += ['--', '-j2']

        env = os.environ.copy()
        env['CXXFLAGS'] = '{} -DVERSION_INFO=\\"{}\\"'.format(env.get('CXXFLAGS', ''),
                                                              self.distribution.get_version())

        if 'LIBOSMIUM_PREFIX' in env:
            cmake_args += ['-DOSMIUM_INCLUDE_DIR={}/include'.format(env['LIBOSMIUM_PREFIX'])]
        elif os.path.exists(os.path.join(BASEDIR, 'contrib', 'libosmium', 'include', 'osmium', 'version.hpp')):
            cmake_args += ['-DOSMIUM_INCLUDE_DIR={}/contrib/libosmium/include'.format(BASEDIR)]

        if 'PROTOZERO_PREFIX' in env:
            cmake_args += ['-DPROTOZERO_INCLUDE_DIR={}/include'.format(env['PROTOZERO_PREFIX'])]
        elif os.path.exists(os.path.join(BASEDIR, 'contrib', 'protozero', 'include', 'protozero', 'version.hpp')):
            cmake_args += ['-DPROTOZERO_INCLUDE_DIR={}/contrib/protozero/include'.format(BASEDIR)]

        if 'PYBIND11_PREFIX' in env:
            cmake_args += ['-DPYBIND11_PREFIX={}'.format(env['PYBIND11_PREFIX'])]
        elif os.path.exists(os.path.join(BASEDIR, 'contrib', 'pybind11')):
            cmake_args += ['-DPYBIND11_PREFIX={}/contrib/pybind11'.format(BASEDIR)]

        if 'BOOST_PREFIX' in env:
            cmake_args += ['-DBOOST_ROOT={}'.format(env['BOOST_PREFIX'])]

        if not os.path.exists(self.build_temp):
            os.makedirs(self.build_temp)
        subprocess.check_call(['cmake', ext.sourcedir] + cmake_args, cwd=self.build_temp, env=env)
        subprocess.check_call(['cmake', '--build', '.'] + build_args, cwd=self.build_temp)

versions = get_versions()

with open('README.rst', 'r') as descfile:
    long_description = descfile.read()

setup(
    name='osmium',
    version=versions['pyosmium_release'],
    description='Python bindings for libosmium, the data processing library for OSM data',
    long_description=long_description,
    author='Sarah Hoffmann',
    author_email='lonvia@denofr.de',
    maintainer='Sarah Hoffmann',
    maintainer_email='lonvia@denofr.de',
    download_url='https://github.com/osmcode/pyosmium',
    url='http://osmcode.org/pyosmium',
    keywords=["OSM", "OpenStreetMap", "Osmium"],
    license='BSD',
    scripts=['tools/pyosmium-get-changes', 'tools/pyosmium-up-to-date'],
    classifiers = [
        "Development Status :: 4 - Beta",
        "Intended Audience :: Developers",
        "License :: OSI Approved :: BSD License",
        "Programming Language :: Python :: 2.7",
        "Programming Language :: Python :: 3.3",
        "Programming Language :: Python :: 3.4",
        "Programming Language :: Python :: 3.5",
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: Implementation :: CPython",
        "Programming Language :: C++",
        ],

    ext_modules=[CMakeExtension('cmake_example')],
    packages = ['osmium', 'osmium/osm', 'osmium/replication'],
    package_dir = {'' : 'src'},
    cmdclass=dict(build_ext=CMakeBuild, sdist=Pyosmium_sdist),
    zip_safe=False,
)



from nose.tools import *
import unittest
import os
import json

from helpers import create_osm_file, osmobj, HandlerTestBase, HandlerFunction
import osmium as o
import osmium.geom

wkbfab = o.geom.WKBFactory()

class NodeGeomCollector(object):

    def __init__(self, fab):
        self.fab = fab
        self.geoms = []

    def __call__(self, n):
        self.geoms.append(self.fab.create_point(n))

    def run(self):
        data = [osmobj('N', id=1, lat=28.0, lon=-23.3)]
        HandlerFunction(node=self).run(data)

        assert_equals(1, len(self.geoms))
        return self.geoms[0]

def test_wkb_create_node():
    c = NodeGeomCollector(o.geom.WKBFactory())
    wkb = c.run()
    if wkb.startswith('01'):
        assert_true(wkb.startswith('0101000000'))
    else:
        assert_true(wkb.startswith('00'))

def test_wkt_create_node():
    c = NodeGeomCollector(o.geom.WKTFactory())
    wkt = c.run()
    assert_true(wkt.startswith('POINT('))

def test_geojson_create_node():
    c = NodeGeomCollector(o.geom.GeoJSONFactory())
    geom = c.run()
    geom = json.loads(geom)
    assert_equal(geom['type'], 'Point')


class WayGeomCollector(object):

    def __init__(self, fab):
        self.fab = fab
        self.geoms = []

    def __call__(self, w):
        self.geoms.append(self.fab.create_linestring(w))
        self.geoms.append(self.fab.create_linestring(w,
                                           direction=o.geom.direction.BACKWARD))
        self.geoms.append(self.fab.create_linestring(w,
                                           use_nodes=o.geom.use_nodes.ALL))

    def run(self):
        data = [osmobj('N', id=1, lat=0, lon=0),
                osmobj('N', id=2, lat=0, lon=1),
                osmobj('N', id=3, lat=1, lon=0),
                osmobj('W', id=1, nodes = [1,2,3])]
        HandlerFunction(way=self).run(data, apply_locations=True)

        assert_equals(3, len(self.geoms))
        return self.geoms

def test_wkb_create_way():
    c = WayGeomCollector(o.geom.WKBFactory())
    for wkb in c.run():
        if wkb.startswith('01'):
            assert_true(wkb.startswith('0102000000030'), "wkb: " + wkb)
        else:
            assert_true(wkb.startswith('00'))

def test_wkt_create_way():
    c = WayGeomCollector(o.geom.WKTFactory())
    for wkt in c.run():
        assert_true(wkt.startswith('LINESTRING('))

def test_geojson_create_way():
    c = WayGeomCollector(o.geom.GeoJSONFactory())
    for geom in c.run():
        geom = json.loads(geom)
        assert_equal(geom['type'], 'LineString')


class PolyGeomCollector(object):

    def __init__(self, fab):
        self.fab = fab
        self.geoms = []

    def __call__(self, n):
        self.geoms.append(self.fab.create_multipolygon(n))

    def run(self):
        data = [osmobj('N', id=1, lat=0, lon=0),
                osmobj('N', id=2, lat=0, lon=1),
                osmobj('N', id=3, lat=1, lon=0),
                osmobj('W', id=23,
                       nodes = [1,2,3,1], tags = { "area" : "yes" })]
        HandlerFunction(area=self).run(data, apply_locations=True)

        assert_equals(1, len(self.geoms))
        return self.geoms[0]

def test_wkb_create_poly():
    c = PolyGeomCollector(o.geom.WKBFactory())
    wkb = c.run()
    if wkb.startswith('01'):
        assert_true(wkb.startswith('010600000001'), "wkb: " + wkb)
    else:
        assert_true(wkb.startswith('00'))

def test_wkt_create_poly():
    c = PolyGeomCollector(o.geom.WKTFactory())
    wkt = c.run()
    assert_true(wkt.startswith('MULTIPOLYGON('))

def test_geojson_create_poly():
    c = PolyGeomCollector(o.geom.GeoJSONFactory())
    geom = c.run()
    geom = json.loads(geom)
    assert_equal(geom['type'], 'MultiPolygon')


def test_lonlat_to_mercator():
    c = o.geom.lonlat_to_mercator(o.geom.Coordinates(3.4,-7.3))
    assert_almost_equals(c.x, 378486.2686971, 5)
    assert_almost_equals(c.y, -814839.8325696, 5)

def test_mercator_lonlat():
    c = o.geom.mercator_to_lonlat(o.geom.Coordinates(0.03,10.2))
    assert_almost_equals(c.x, 0.00000026, 5)
    assert_almost_equals(c.y, 0.00009162, 5)

def test_coordinate_from_location():
    c = o.geom.Coordinates(o.osm.Location(10.0, -3.0))
    assert_almost_equals(c.x, 10.0)
    assert_almost_equals(c.y, -3.0)

def test_haversine():
    data = [osmobj('N', id=1, lat=0, lon=0),
                osmobj('N', id=2, lat=0, lon=1),
                osmobj('N', id=3, lat=1, lon=0),
                osmobj('W', id=1, nodes = [1,2,3])]

    results = []
    def call_haversine(w):
        results.append(o.geom.haversine_distance(w.nodes))
    HandlerFunction(way=call_haversine).run(data, apply_locations=True)

    assert_equal(1, len(results))
    assert_almost_equals(268520, results[0], 0)
